@@ -1,17 +1,27 @@
-'''Test cases for db_pool
-'''
 from __future__ import print_function
-
-import sys
 import os
+import sys
 import traceback
-from unittest import TestCase, main
 
-from tests import mock, skipped, skip_unless, skip_with_pyevent, get_database_auth
-from eventlet import event
 from eventlet import db_pool
 from eventlet.support import six
 import eventlet
+import eventlet.tpool
+import tests
+import tests.mock
+
+psycopg2 = None
+try:
+    import psycopg2
+    import psycopg2.extensions
+except ImportError:
+    pass
+
+MySQLdb = None
+try:
+    import MySQLdb
+except ImportError:
+    pass
 
 
 class DBTester(object):
@@ -116,14 +126,6 @@ class DBConnectionPool(DBTester):
         assert self.pool.free() == 1
         self.assertRaises(AttributeError, self.connection.cursor)
 
-    @skipped
-    def test_deletion_does_a_put(self):
-        # doing a put on del causes some issues if __del__ is called in the
-        # main coroutine, so, not doing that for now
-        assert self.pool.free() == 0
-        self.connection = None
-        assert self.pool.free() == 1
-
     def test_put_doesnt_double_wrap(self):
         self.pool.put(self.connection)
         conn = self.pool.get()
@@ -149,7 +151,7 @@ class DBConnectionPool(DBTester):
         curs = conn.cursor()
         results = []
         SHORT_QUERY = "select * from test_table"
-        evt = event.Event()
+        evt = eventlet.Event()
 
         def a_query():
             self.assert_cursor_works(curs)
@@ -213,45 +215,6 @@ class DBConnectionPool(DBTester):
             conn.commit()
             self.pool.put(conn)
 
-    @skipped
-    def test_two_simultaneous_connections(self):
-        # timing-sensitive test, disabled until we come up with a better
-        # way to do this
-        self.pool = self.create_pool(max_size=2)
-        conn = self.pool.get()
-        self.set_up_dummy_table(conn)
-        self.fill_up_table(conn)
-        curs = conn.cursor()
-        conn2 = self.pool.get()
-        self.set_up_dummy_table(conn2)
-        self.fill_up_table(conn2)
-        curs2 = conn2.cursor()
-        results = []
-        LONG_QUERY = "select * from test_table"
-        SHORT_QUERY = "select * from test_table where row_id <= 20"
-
-        evt = event.Event()
-
-        def long_running_query():
-            self.assert_cursor_works(curs)
-            curs.execute(LONG_QUERY)
-            results.append(1)
-            evt.send()
-        evt2 = event.Event()
-
-        def short_running_query():
-            self.assert_cursor_works(curs2)
-            curs2.execute(SHORT_QUERY)
-            results.append(2)
-            evt2.send()
-
-        eventlet.spawn(long_running_query)
-        eventlet.spawn(short_running_query)
-        evt.wait()
-        evt2.wait()
-        results.sort()
-        self.assertEqual([1, 2], results)
-
     def test_clear(self):
         self.pool = self.create_pool()
         self.pool.put(self.connection)
@@ -318,80 +281,6 @@ class DBConnectionPool(DBTester):
         self.connection.close()
         self.assertEqual(len(self.pool.free_items), 0)
 
-    @skipped
-    def test_max_idle(self):
-        # This test is timing-sensitive.  Rename the function without
-        # the "dont" to run it, but beware that it could fail or take
-        # a while.
-
-        self.pool = self.create_pool(max_size=2, max_idle=0.02)
-        self.connection = self.pool.get()
-        self.connection.close()
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0.01)  # not long enough to trigger the idle timeout
-        self.assertEqual(len(self.pool.free_items), 1)
-        self.connection = self.pool.get()
-        self.connection.close()
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0.01)  # idle timeout should have fired but done nothing
-        self.assertEqual(len(self.pool.free_items), 1)
-        self.connection = self.pool.get()
-        self.connection.close()
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0.03)  # long enough to trigger idle timeout for real
-        self.assertEqual(len(self.pool.free_items), 0)
-
-    @skipped
-    def test_max_idle_many(self):
-        # This test is timing-sensitive.  Rename the function without
-        # the "dont" to run it, but beware that it could fail or take
-        # a while.
-
-        self.pool = self.create_pool(max_size=2, max_idle=0.02)
-        self.connection, conn2 = self.pool.get(), self.pool.get()
-        self.connection.close()
-        eventlet.sleep(0.01)
-        self.assertEqual(len(self.pool.free_items), 1)
-        conn2.close()
-        self.assertEqual(len(self.pool.free_items), 2)
-        eventlet.sleep(0.02)  # trigger cleanup of conn1 but not conn2
-        self.assertEqual(len(self.pool.free_items), 1)
-
-    @skipped
-    def test_max_age(self):
-        # This test is timing-sensitive.  Rename the function without
-        # the "dont" to run it, but beware that it could fail or take
-        # a while.
-
-        self.pool = self.create_pool(max_size=2, max_age=0.05)
-        self.connection = self.pool.get()
-        self.connection.close()
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0.01)  # not long enough to trigger the age timeout
-        self.assertEqual(len(self.pool.free_items), 1)
-        self.connection = self.pool.get()
-        self.connection.close()
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0.05)  # long enough to trigger age timeout
-        self.assertEqual(len(self.pool.free_items), 0)
-
-    @skipped
-    def test_max_age_many(self):
-        # This test is timing-sensitive.  Rename the function without
-        # the "dont" to run it, but beware that it could fail or take
-        # a while.
-
-        self.pool = self.create_pool(max_size=2, max_age=0.15)
-        self.connection, conn2 = self.pool.get(), self.pool.get()
-        self.connection.close()
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0)  # not long enough to trigger the age timeout
-        self.assertEqual(len(self.pool.free_items), 1)
-        eventlet.sleep(0.2)  # long enough to trigger age timeout
-        self.assertEqual(len(self.pool.free_items), 0)
-        conn2.close()  # should not be added to the free items
-        self.assertEqual(len(self.pool.free_items), 0)
-
     def test_waiters_get_woken(self):
         # verify that when there's someone waiting on an empty pool
         # and someone puts an immediately-closed connection back in
@@ -403,7 +292,7 @@ class DBConnectionPool(DBTester):
         self.connection = self.pool.get()
         self.assertEqual(self.pool.free(), 0)
         self.assertEqual(self.pool.waiting(), 0)
-        e = event.Event()
+        e = eventlet.Event()
 
         def retrieve(pool, ev):
             c = pool.get()
@@ -421,29 +310,6 @@ class DBConnectionPool(DBTester):
         self.assertEqual(self.pool.waiting(), 0)
         self.pool.put(conn)
 
-    @skipped
-    def test_0_straight_benchmark(self):
-        """ Benchmark; don't run unless you want to wait a while."""
-        import time
-        iterations = 20000
-        c = self.connection.cursor()
-        self.connection.commit()
-
-        def bench(c):
-            for i in six.moves.range(iterations):
-                c.execute('select 1')
-
-        bench(c)  # warm-up
-        results = []
-        for i in range(3):
-            start = time.time()
-            bench(c)
-            end = time.time()
-            results.append(end - start)
-
-        print("\n%u iterations took an average of %f seconds, (%s) in %s\n" % (
-            iterations, sum(results) / len(results), results, type(self)))
-
     def test_raising_create(self):
         # if the create() method raises an exception the pool should
         # not lose any connections
@@ -453,7 +319,8 @@ class DBConnectionPool(DBTester):
 
 
 class DummyConnection(object):
-    pass
+    def rollback(self):
+        pass
 
 
 class DummyDBModule(object):
@@ -480,14 +347,13 @@ class TpoolConnectionPool(DBConnectionPool):
             connect_timeout=connect_timeout,
             **self._auth)
 
-    @skip_with_pyevent
+    @tests.skip_with_pyevent
     def setUp(self):
         super(TpoolConnectionPool, self).setUp()
 
     def tearDown(self):
         super(TpoolConnectionPool, self).tearDown()
-        from eventlet import tpool
-        tpool.killall()
+        eventlet.tpool.killall()
 
 
 class RawConnectionPool(DBConnectionPool):
@@ -505,77 +371,88 @@ class RawConnectionPool(DBConnectionPool):
             **self._auth)
 
 
-class TestRawConnectionPool(TestCase):
-    def test_issue_125(self):
-        # pool = self.create_pool(min_size=3, max_size=5)
-        pool = db_pool.RawConnectionPool(
-            DummyDBModule(),
-            dsn="dbname=test user=jessica port=5433",
-            min_size=3, max_size=5)
-        conn = pool.get()
+def test_raw_pool_issue_125():
+    # pool = self.create_pool(min_size=3, max_size=5)
+    pool = db_pool.RawConnectionPool(
+        DummyDBModule(),
+        dsn="dbname=test user=jessica port=5433",
+        min_size=3, max_size=5)
+    conn = pool.get()
+    pool.put(conn)
+
+
+def test_raw_pool_custom_cleanup_ok():
+    cleanup_mock = tests.mock.Mock()
+    pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup_mock)
+    conn = pool.get()
+    pool.put(conn)
+    assert cleanup_mock.call_count == 1
+
+    with pool.item() as conn:
+        pass
+    assert cleanup_mock.call_count == 2
+
+
+def test_raw_pool_custom_cleanup_arg_error():
+    cleanup_mock = tests.mock.Mock(side_effect=NotImplementedError)
+    pool = db_pool.RawConnectionPool(DummyDBModule())
+    conn = pool.get()
+    pool.put(conn, cleanup=cleanup_mock)
+    assert cleanup_mock.call_count == 1
+
+    with pool.item(cleanup=cleanup_mock):
+        pass
+    assert cleanup_mock.call_count == 2
+
+
+def test_raw_pool_custom_cleanup_fatal():
+    state = [0]
+
+    def cleanup(conn):
+        state[0] += 1
+        raise KeyboardInterrupt
+
+    pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup)
+    conn = pool.get()
+    try:
         pool.put(conn)
-
-    def test_custom_cleanup_ok(self):
-        cleanup_mock = mock.Mock()
-        pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup_mock)
-        conn = pool.get()
-        pool.put(conn)
-        assert cleanup_mock.call_count == 1
-
-        with pool.item() as conn:
-            pass
-        assert cleanup_mock.call_count == 2
-
-    def test_custom_cleanup_arg_error(self):
-        cleanup_mock = mock.Mock(side_effect=NotImplementedError)
-        pool = db_pool.RawConnectionPool(DummyDBModule())
-        conn = pool.get()
-        pool.put(conn, cleanup=cleanup_mock)
-        assert cleanup_mock.call_count == 1
-
-        with pool.item(cleanup=cleanup_mock):
-            pass
-        assert cleanup_mock.call_count == 2
-
-    def test_custom_cleanup_fatal(self):
-        state = [0]
-
-        def cleanup(conn):
-            state[0] += 1
-            raise KeyboardInterrupt
-
-        pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup)
-        conn = pool.get()
-        try:
-            pool.put(conn)
-        except KeyboardInterrupt:
-            pass
-        else:
-            assert False, 'Expected KeyboardInterrupt'
-        assert state[0] == 1
+    except KeyboardInterrupt:
+        pass
+    else:
+        assert False, 'Expected KeyboardInterrupt'
+    assert state[0] == 1
 
 
-get_auth = get_database_auth
+def test_raw_pool_clear_update_current_size():
+    # https://github.com/eventlet/eventlet/issues/139
+    # BaseConnectionPool.clear does not update .current_size.
+    # That leads to situation when new connections could not be created.
+    pool = db_pool.RawConnectionPool(DummyDBModule())
+    pool.get().close()
+    assert pool.current_size == 1
+    assert len(pool.free_items) == 1
+    pool.clear()
+    assert pool.current_size == 0
+    assert len(pool.free_items) == 0
 
 
 def mysql_requirement(_f):
     verbose = os.environ.get('eventlet_test_mysql_verbose')
-    try:
-        import MySQLdb
-        try:
-            auth = get_auth()['MySQLdb'].copy()
-            MySQLdb.connect(**auth)
-            return True
-        except MySQLdb.OperationalError:
-            if verbose:
-                print(">> Skipping mysql tests, error when connecting:", file=sys.stderr)
-                traceback.print_exc()
-            return False
-    except ImportError:
+    if MySQLdb is None:
         if verbose:
             print(">> Skipping mysql tests, MySQLdb not importable", file=sys.stderr)
         return False
 
+    try:
+        auth = tests.get_database_auth()['MySQLdb'].copy()
+        MySQLdb.connect(**auth)
+        return True
+    except MySQLdb.OperationalError:
+        if verbose:
+            print(">> Skipping mysql tests, error when connecting:", file=sys.stderr)
+            traceback.print_exc()
+        return False
+
 
 class MysqlConnectionPool(object):
     dummy_table_sql = """CREATE TEMPORARY TABLE test_table
@@ -591,11 +468,10 @@ class MysqlConnectionPool(object):
         created TIMESTAMP
         ) ENGINE=InnoDB;"""
 
-    @skip_unless(mysql_requirement)
+    @tests.skip_unless(mysql_requirement)
     def setUp(self):
-        import MySQLdb
         self._dbmodule = MySQLdb
-        self._auth = get_auth()['MySQLdb']
+        self._auth = tests.get_database_auth()['MySQLdb']
         super(MysqlConnectionPool, self).setUp()
 
     def tearDown(self):
@@ -621,28 +497,27 @@ class MysqlConnectionPool(object):
         del db
 
 
-class Test01MysqlTpool(MysqlConnectionPool, TpoolConnectionPool, TestCase):
+class Test01MysqlTpool(MysqlConnectionPool, TpoolConnectionPool, tests.LimitedTestCase):
     __test__ = True
 
 
-class Test02MysqlRaw(MysqlConnectionPool, RawConnectionPool, TestCase):
+class Test02MysqlRaw(MysqlConnectionPool, RawConnectionPool, tests.LimitedTestCase):
     __test__ = True
 
 
 def postgres_requirement(_f):
-    try:
-        import psycopg2
-        try:
-            auth = get_auth()['psycopg2'].copy()
-            psycopg2.connect(**auth)
-            return True
-        except psycopg2.OperationalError:
-            print("Skipping postgres tests, error when connecting")
-            return False
-    except ImportError:
+    if psycopg2 is None:
         print("Skipping postgres tests, psycopg2 not importable")
         return False
 
+    try:
+        auth = tests.get_database_auth()['psycopg2'].copy()
+        psycopg2.connect(**auth)
+        return True
+    except psycopg2.OperationalError:
+        print("Skipping postgres tests, error when connecting")
+        return False
+
 
 class Psycopg2ConnectionPool(object):
     dummy_table_sql = """CREATE TEMPORARY TABLE test_table
@@ -657,11 +532,10 @@ class Psycopg2ConnectionPool(object):
         created TIMESTAMP
         );"""
 
-    @skip_unless(postgres_requirement)
+    @tests.skip_unless(postgres_requirement)
     def setUp(self):
-        import psycopg2
         self._dbmodule = psycopg2
-        self._auth = get_auth()['psycopg2']
+        self._auth = tests.get_database_auth()['psycopg2']
         super(Psycopg2ConnectionPool, self).setUp()
 
     def tearDown(self):
@@ -694,7 +568,7 @@ class Psycopg2ConnectionPool(object):
         conn.close()
 
 
-class TestPsycopg2Base(TestCase):
+class TestPsycopg2Base(tests.LimitedTestCase):
     __test__ = False
 
     def test_cursor_works_as_context_manager(self):
@@ -703,6 +577,9 @@ class TestPsycopg2Base(TestCase):
             row = c.fetchone()
             assert row == (1,)
 
+    def test_set_isolation_level(self):
+        self.connection.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
+
 
 class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestPsycopg2Base):
     __test__ = True
@@ -710,7 +587,3 @@ class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestPsyco
 
 class Test02Psycopg2Raw(Psycopg2ConnectionPool, RawConnectionPool, TestPsycopg2Base):
     __test__ = True
-
-
-if __name__ == '__main__':
-    main()
