@@ -1,17 +1,17 @@
 import errno
 import socket
+import sys
 
 import eventlet
 from eventlet import event
 from eventlet import greenio
-from eventlet import wsgi
 from eventlet.green import httplib
-from eventlet.green import urllib2
+from eventlet.support import six
 from eventlet.websocket import WebSocket, WebSocketWSGI
 
-from tests import mock, LimitedTestCase, certificate_file, private_key_file
-from tests import skip_if_no_ssl
-from tests.wsgi_test import _TestBase
+import tests
+from tests import mock
+import tests.wsgi_test
 
 
 # demo app
@@ -35,120 +35,115 @@ def handle(ws):
 wsapp = WebSocketWSGI(handle)
 
 
-class TestWebSocket(_TestBase):
+class TestWebSocket(tests.wsgi_test._TestBase):
     TEST_TIMEOUT = 5
 
     def set_site(self):
         self.site = wsapp
 
     def test_incorrect_headers(self):
-        def raiser():
-            try:
-                urllib2.urlopen("http://localhost:%s/echo" % self.port)
-            except urllib2.HTTPError as e:
-                self.assertEqual(e.code, 400)
-                raise
-        self.assertRaises(urllib2.HTTPError, raiser)
+        http = httplib.HTTPConnection(*self.server_addr)
+        http.request("GET", "/echo")
+        response = http.getresponse()
+        assert response.status == 400
 
     def test_incomplete_headers_75(self):
         headers = dict(kv.split(': ') for kv in [
             "Upgrade: WebSocket",
             # NOTE: intentionally no connection header
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "WebSocket-Protocol: ws",
         ])
-        http = httplib.HTTPConnection('localhost', self.port)
+        http = httplib.HTTPConnection(*self.server_addr)
         http.request("GET", "/echo", headers=headers)
         resp = http.getresponse()
 
         self.assertEqual(resp.status, 400)
         self.assertEqual(resp.getheader('connection'), 'close')
-        self.assertEqual(resp.read(), '')
+        self.assertEqual(resp.read(), b'')
 
     def test_incomplete_headers_76(self):
         # First test: Missing Connection:
         headers = dict(kv.split(': ') for kv in [
             "Upgrade: WebSocket",
             # NOTE: intentionally no connection header
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
         ])
-        http = httplib.HTTPConnection('localhost', self.port)
+        http = httplib.HTTPConnection(*self.server_addr)
         http.request("GET", "/echo", headers=headers)
         resp = http.getresponse()
 
         self.assertEqual(resp.status, 400)
         self.assertEqual(resp.getheader('connection'), 'close')
-        self.assertEqual(resp.read(), '')
+        self.assertEqual(resp.read(), b'')
 
         # Now, miss off key2
         headers = dict(kv.split(': ') for kv in [
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             # NOTE: Intentionally no Key2 header
         ])
-        http = httplib.HTTPConnection('localhost', self.port)
+        http = httplib.HTTPConnection(*self.server_addr)
         http.request("GET", "/echo", headers=headers)
         resp = http.getresponse()
 
         self.assertEqual(resp.status, 400)
         self.assertEqual(resp.getheader('connection'), 'close')
-        self.assertEqual(resp.read(), '')
+        self.assertEqual(resp.read(), b'')
 
     def test_correct_upgrade_request_75(self):
         connect = [
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "WebSocket-Protocol: ws",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n'))
         result = sock.recv(1024)
         # The server responds the correct Websocket handshake
-        self.assertEqual(result, '\r\n'.join([
+        self.assertEqual(result, six.b('\r\n'.join([
             'HTTP/1.1 101 Web Socket Protocol Handshake',
             'Upgrade: WebSocket',
             'Connection: Upgrade',
-            'WebSocket-Origin: http://localhost:%s' % self.port,
-            'WebSocket-Location: ws://localhost:%s/echo\r\n\r\n' % self.port,
-        ]))
+            'WebSocket-Origin: http://%s:%s' % self.server_addr,
+            'WebSocket-Location: ws://%s:%s/echo\r\n\r\n' % self.server_addr,
+        ])))
 
     def test_correct_upgrade_request_76(self):
         connect = [
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         result = sock.recv(1024)
         # The server responds the correct Websocket handshake
-        self.assertEqual(result, '\r\n'.join([
+        self.assertEqual(result, six.b('\r\n'.join([
             'HTTP/1.1 101 WebSocket Protocol Handshake',
             'Upgrade: WebSocket',
             'Connection: Upgrade',
-            'Sec-WebSocket-Origin: http://localhost:%s' % self.port,
+            'Sec-WebSocket-Origin: http://%s:%s' % self.server_addr,
             'Sec-WebSocket-Protocol: ws',
-            'Sec-WebSocket-Location: ws://localhost:%s/echo\r\n\r\n8jKS\'y:G*Co,Wxa-' % self.port,
-        ]))
+            'Sec-WebSocket-Location: ws://%s:%s/echo\r\n\r\n8jKS\'y:G*Co,Wxa-' % self.server_addr,
+        ])))
 
     def test_query_string(self):
         # verify that the query string comes out the other side unscathed
@@ -156,25 +151,25 @@ class TestWebSocket(_TestBase):
             "GET /echo?query_string HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         result = sock.recv(1024)
-        self.assertEqual(result, '\r\n'.join([
+        self.assertEqual(result, six.b('\r\n'.join([
             'HTTP/1.1 101 WebSocket Protocol Handshake',
             'Upgrade: WebSocket',
             'Connection: Upgrade',
-            'Sec-WebSocket-Origin: http://localhost:%s' % self.port,
+            'Sec-WebSocket-Origin: http://%s:%s' % self.server_addr,
             'Sec-WebSocket-Protocol: ws',
-            'Sec-WebSocket-Location: ws://localhost:%s/echo?query_string\r\n\r\n8jKS\'y:G*Co,Wxa-' % self.port,
-        ]))
+            'Sec-WebSocket-Location: '
+            'ws://%s:%s/echo?query_string\r\n\r\n8jKS\'y:G*Co,Wxa-' % self.server_addr,
+        ])))
 
     def test_empty_query_string(self):
         # verify that a single trailing ? doesn't get nuked
@@ -182,48 +177,46 @@ class TestWebSocket(_TestBase):
             "GET /echo? HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         result = sock.recv(1024)
-        self.assertEqual(result, '\r\n'.join([
+        self.assertEqual(result, six.b('\r\n'.join([
             'HTTP/1.1 101 WebSocket Protocol Handshake',
             'Upgrade: WebSocket',
             'Connection: Upgrade',
-            'Sec-WebSocket-Origin: http://localhost:%s' % self.port,
+            'Sec-WebSocket-Origin: http://%s:%s' % self.server_addr,
             'Sec-WebSocket-Protocol: ws',
-            'Sec-WebSocket-Location: ws://localhost:%s/echo?\r\n\r\n8jKS\'y:G*Co,Wxa-' % self.port,
-        ]))
+            'Sec-WebSocket-Location: ws://%s:%s/echo?\r\n\r\n8jKS\'y:G*Co,Wxa-' % self.server_addr,
+        ])))
 
     def test_sending_messages_to_websocket_75(self):
         connect = [
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "WebSocket-Protocol: ws",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n'))
         sock.recv(1024)
-        sock.sendall('\x00hello\xFF')
+        sock.sendall(b'\x00hello\xFF')
         result = sock.recv(1024)
-        self.assertEqual(result, '\x00hello\xff')
-        sock.sendall('\x00start')
+        self.assertEqual(result, b'\x00hello\xff')
+        sock.sendall(b'\x00start')
         eventlet.sleep(0.001)
-        sock.sendall(' end\xff')
+        sock.sendall(b' end\xff')
         result = sock.recv(1024)
-        self.assertEqual(result, '\x00start end\xff')
+        self.assertEqual(result, b'\x00start end\xff')
         sock.shutdown(socket.SHUT_RDWR)
         sock.close()
         eventlet.sleep(0.01)
@@ -233,25 +226,24 @@ class TestWebSocket(_TestBase):
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         sock.recv(1024)
-        sock.sendall('\x00hello\xFF')
+        sock.sendall(b'\x00hello\xFF')
         result = sock.recv(1024)
-        self.assertEqual(result, '\x00hello\xff')
-        sock.sendall('\x00start')
+        self.assertEqual(result, b'\x00hello\xff')
+        sock.sendall(b'\x00start')
         eventlet.sleep(0.001)
-        sock.sendall(' end\xff')
+        sock.sendall(b' end\xff')
         result = sock.recv(1024)
-        self.assertEqual(result, '\x00start end\xff')
+        self.assertEqual(result, b'\x00start end\xff')
         sock.shutdown(socket.SHUT_RDWR)
         sock.close()
         eventlet.sleep(0.01)
@@ -261,48 +253,46 @@ class TestWebSocket(_TestBase):
             "GET /range HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "WebSocket-Protocol: ws",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n'))
         resp = sock.recv(1024)
-        headers, result = resp.split('\r\n\r\n')
-        msgs = [result.strip('\x00\xff')]
+        headers, result = resp.split(b'\r\n\r\n')
+        msgs = [result.strip(b'\x00\xff')]
         cnt = 10
         while cnt:
-            msgs.append(sock.recv(20).strip('\x00\xff'))
+            msgs.append(sock.recv(20).strip(b'\x00\xff'))
             cnt -= 1
         # Last item in msgs is an empty string
-        self.assertEqual(msgs[:-1], ['msg %d' % i for i in range(10)])
+        self.assertEqual(msgs[:-1], [six.b('msg %d' % i) for i in range(10)])
 
     def test_getting_messages_from_websocket_76(self):
         connect = [
             "GET /range HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
+        sock = eventlet.connect(self.server_addr)
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         resp = sock.recv(1024)
-        headers, result = resp.split('\r\n\r\n')
-        msgs = [result[16:].strip('\x00\xff')]
+        headers, result = resp.split(b'\r\n\r\n')
+        msgs = [result[16:].strip(b'\x00\xff')]
         cnt = 10
         while cnt:
-            msgs.append(sock.recv(20).strip('\x00\xff'))
+            msgs.append(sock.recv(20).strip(b'\x00\xff'))
             cnt -= 1
         # Last item in msgs is an empty string
-        self.assertEqual(msgs[:-1], ['msg %d' % i for i in range(10)])
+        self.assertEqual(msgs[:-1], [six.b('msg %d' % i) for i in range(10)])
 
     def test_breaking_the_connection_75(self):
         error_detected = [False]
@@ -324,13 +314,12 @@ class TestWebSocket(_TestBase):
             "GET /range HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "WebSocket-Protocol: ws",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n'))
         sock.recv(1024)  # get the headers
         sock.close()  # close while the app is running
         done_with_request.wait()
@@ -356,15 +345,14 @@ class TestWebSocket(_TestBase):
             "GET /range HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         sock.recv(1024)  # get the headers
         sock.close()  # close while the app is running
         done_with_request.wait()
@@ -390,17 +378,16 @@ class TestWebSocket(_TestBase):
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         sock.recv(1024)  # get the headers
-        sock.sendall('\xff\x00')  # "Close the connection" packet.
+        sock.sendall(b'\xff\x00')  # "Close the connection" packet.
         done_with_request.wait()
         assert not error_detected[0]
 
@@ -424,17 +411,16 @@ class TestWebSocket(_TestBase):
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         sock.recv(1024)  # get the headers
-        sock.sendall('\xef\x00')  # Weird packet.
+        sock.sendall(b'\xef\x00')  # Weird packet.
         done_with_request.wait()
         assert error_detected[0]
 
@@ -443,20 +429,18 @@ class TestWebSocket(_TestBase):
             "GET / HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         resp = sock.recv(1024)
-        headers, result = resp.split('\r\n\r\n')
+        headers, result = resp.split(b'\r\n\r\n')
         # The remote server should have immediately closed the connection.
-        self.assertEqual(result[16:], '\xff\x00')
+        self.assertEqual(result[16:], b'\xff\x00')
 
     def test_app_socket_errors_75(self):
         error_detected = [False]
@@ -478,13 +462,12 @@ class TestWebSocket(_TestBase):
             "GET /error HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "WebSocket-Protocol: ws",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n'))
         sock.recv(1024)
         done_with_request.wait()
         assert error_detected[0]
@@ -509,65 +492,92 @@ class TestWebSocket(_TestBase):
             "GET /error HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.connect(
-            ('localhost', self.port))
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
         sock.recv(1024)
         done_with_request.wait()
         assert error_detected[0]
 
+    def test_close_idle(self):
+        pool = eventlet.GreenPool()
+        # use log=stderr when test runner can capture it
+        self.spawn_server(custom_pool=pool, log=sys.stdout)
+        connect = (
+            'GET /echo HTTP/1.1',
+            'Upgrade: WebSocket',
+            'Connection: Upgrade',
+            'Host: %s:%s' % self.server_addr,
+            'Origin: http://%s:%s' % self.server_addr,
+            'Sec-WebSocket-Protocol: ws',
+            'Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5',
+            'Sec-WebSocket-Key2: 12998 5 Y3 1  .P00',
+        )
+        sock = eventlet.connect(self.server_addr)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
+        sock.recv(1024)
+        sock.sendall(b'\x00hello\xff')
+        result = sock.recv(1024)
+        assert result, b'\x00hello\xff'
+        self.killer.kill(KeyboardInterrupt)
+        with eventlet.Timeout(1):
+            pool.waitall()
 
-class TestWebSocketSSL(_TestBase):
+
+class TestWebSocketSSL(tests.wsgi_test._TestBase):
     def set_site(self):
         self.site = wsapp
 
-    @skip_if_no_ssl
+    @tests.skip_if_no_ssl
     def test_ssl_sending_messages(self):
         s = eventlet.wrap_ssl(eventlet.listen(('localhost', 0)),
-                              certfile=certificate_file,
-                              keyfile=private_key_file,
+                              certfile=tests.certificate_file,
+                              keyfile=tests.private_key_file,
                               server_side=True)
         self.spawn_server(sock=s)
         connect = [
             "GET /echo HTTP/1.1",
             "Upgrade: WebSocket",
             "Connection: Upgrade",
-            "Host: localhost:%s" % self.port,
-            "Origin: http://localhost:%s" % self.port,
+            "Host: %s:%s" % self.server_addr,
+            "Origin: http://%s:%s" % self.server_addr,
             "Sec-WebSocket-Protocol: ws",
             "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5",
             "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00",
         ]
-        sock = eventlet.wrap_ssl(eventlet.connect(
-            ('localhost', self.port)))
+        sock = eventlet.wrap_ssl(eventlet.connect(self.server_addr))
 
-        sock.sendall('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U')
-        first_resp = sock.recv(1024)
+        sock.sendall(six.b('\r\n'.join(connect) + '\r\n\r\n^n:ds[4U'))
+        first_resp = b''
+        while b'\r\n\r\n' not in first_resp:
+            first_resp += sock.recv()
+            print('resp now:')
+            print(first_resp)
         # make sure it sets the wss: protocol on the location header
-        loc_line = [x for x in first_resp.split("\r\n")
-                    if x.lower().startswith('sec-websocket-location')][0]
-        self.assert_("wss://localhost" in loc_line,
-                     "Expecting wss protocol in location: %s" % loc_line)
-        sock.sendall('\x00hello\xFF')
+        loc_line = [x for x in first_resp.split(b"\r\n")
+                    if x.lower().startswith(b'sec-websocket-location')][0]
+        expect_wss = ('wss://%s:%s' % self.server_addr).encode()
+        assert expect_wss in loc_line, "Expecting wss protocol in location: %s" % loc_line
+
+        sock.sendall(b'\x00hello\xFF')
         result = sock.recv(1024)
-        self.assertEqual(result, '\x00hello\xff')
-        sock.sendall('\x00start')
+        self.assertEqual(result, b'\x00hello\xff')
+        sock.sendall(b'\x00start')
         eventlet.sleep(0.001)
-        sock.sendall(' end\xff')
+        sock.sendall(b' end\xff')
         result = sock.recv(1024)
-        self.assertEqual(result, '\x00start end\xff')
+        self.assertEqual(result, b'\x00start end\xff')
         greenio.shutdown_safe(sock)
         sock.close()
         eventlet.sleep(0.01)
 
 
-class TestWebSocketObject(LimitedTestCase):
+class TestWebSocketObject(tests.LimitedTestCase):
 
     def setUp(self):
         self.mock_socket = s = mock.Mock()
@@ -579,13 +589,13 @@ class TestWebSocketObject(LimitedTestCase):
 
     def test_recieve(self):
         ws = self.test_ws
-        ws.socket.recv.return_value = '\x00hello\xFF'
+        ws.socket.recv.return_value = b'\x00hello\xFF'
         self.assertEqual(ws.wait(), 'hello')
-        self.assertEqual(ws._buf, '')
+        self.assertEqual(ws._buf, b'')
         self.assertEqual(len(ws._msgs), 0)
-        ws.socket.recv.return_value = ''
+        ws.socket.recv.return_value = b''
         self.assertEqual(ws.wait(), None)
-        self.assertEqual(ws._buf, '')
+        self.assertEqual(ws._buf, b'')
         self.assertEqual(len(ws._msgs), 0)
 
     def test_send_to_ws(self):
